@@ -62,6 +62,11 @@ typedef enum {
 	EOG_SCROLL_VIEW_CURSOR_DRAG
 } EogScrollViewCursor;
 
+/* Drag 'n Drop */
+static GtkTargetEntry target_table[] = {
+	{ "text/uri-list", 0, 0},
+};
+
 /* Private part of the EogScrollView structure */
 struct _EogScrollViewPrivate {
 	/* some widgets we rely on */
@@ -95,13 +100,16 @@ struct _EogScrollViewPrivate {
 	/* Microtile arrays for dirty region.  This represents the dirty region
 	 * for interpolated drawing.
 	 */
-	ArtUta *uta;
+	EogUta *uta;
 
 	/* handler ID for paint idle callback */
 	guint idle_id;
 
-	/* Interpolation type */
-	GdkInterpType interp_type;
+	/* Interpolation type when zoomed in*/
+	GdkInterpType interp_type_in;
+
+	/* Interpolation type when zoomed out*/
+	GdkInterpType interp_type_out;
 
 	/* Scroll wheel zoom */
 	gboolean scroll_wheel_zoom;
@@ -378,7 +386,7 @@ check_scrollbar_visibility (EogScrollView *view, GtkAllocation *alloc)
 	gtk_widget_size_request (priv->vbar, &req);
 	bar_width = req.width;
 
-	eog_debug_message (DEBUG_WINDOW, "Widget Size allocate: %i, %i   Bar: %i, %i\n", 
+	eog_debug_message (DEBUG_WINDOW, "Widget Size allocate: %i, %i   Bar: %i, %i\n",
 			   width, height, bar_width, bar_height);
 
 	hbar_visible = vbar_visible = FALSE;
@@ -423,7 +431,27 @@ is_unity_zoom (EogScrollView *view)
 	return DOUBLE_EQUAL (priv->zoom, 1.0);
 }
 
-/* Returns wether the image is movable, that means if it is larger then 
+/* Returns whether the image is zoomed in */
+static gboolean
+is_zoomed_in (EogScrollView *view)
+{
+	EogScrollViewPrivate *priv;
+
+	priv = view->priv;
+	return priv->zoom - 1.0 > DOUBLE_EQUAL_MAX_DIFF;
+}
+
+/* Returns whether the image is zoomed out */
+static gboolean
+is_zoomed_out (EogScrollView *view)
+{
+	EogScrollViewPrivate *priv;
+
+	priv = view->priv;
+	return DOUBLE_EQUAL_MAX_DIFF + priv->zoom - 1.0 < 0.0;
+}
+
+/* Returns wether the image is movable, that means if it is larger then
  * the actual visible area.
  */
 static gboolean
@@ -477,7 +505,7 @@ get_image_offsets (EogScrollView *view, int *xofs, int *yofs)
  * from the microtile array.
  */
 static void
-pull_rectangle (ArtUta *uta, ArtIRect *rect, int max_width, int max_height)
+pull_rectangle (EogUta *uta, EogIRect *rect, int max_width, int max_height)
 {
 	uta_find_first_glom_rect (uta, rect, max_width, max_height);
 	uta_remove_rect (uta, rect->x0, rect->y0, rect->x1, rect->y1);
@@ -487,15 +515,15 @@ pull_rectangle (ArtUta *uta, ArtIRect *rect, int max_width, int max_height)
  * intersects the dirty rectangle.
  */
 static void
-paint_background (EogScrollView *view, ArtIRect *r, ArtIRect *rect)
+paint_background (EogScrollView *view, EogIRect *r, EogIRect *rect)
 {
 	EogScrollViewPrivate *priv;
-	ArtIRect d;
+	EogIRect d;
 
 	priv = view->priv;
 
-	art_irect_intersect (&d, r, rect);
-	if (!art_irect_empty (&d)) {
+	eog_irect_intersect (&d, r, rect);
+	if (!eog_irect_empty (&d)) {
 		gdk_window_clear_area (GTK_WIDGET (priv->display)->window,
 				       d.x0, d.y0,
 				       d.x1 - d.x0, d.y1 - d.y0);
@@ -504,13 +532,13 @@ paint_background (EogScrollView *view, ArtIRect *r, ArtIRect *rect)
 
 /* Paints a rectangle of the dirty region */
 static void
-paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
+paint_rectangle (EogScrollView *view, EogIRect *rect, GdkInterpType interp_type)
 {
 	EogScrollViewPrivate *priv;
 	int scaled_width, scaled_height;
 	int width, height;
 	int xofs, yofs;
-	ArtIRect r, d;
+	EogIRect r, d;
 	GdkPixbuf *tmp;
 	int check_size;
 	char *str;
@@ -536,7 +564,7 @@ paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
 		paint_background (view, &r, rect);
 		return;
 	}
-	
+
 	/* Compute image offsets with respect to the window */
 
 	if (scaled_width <= width)
@@ -549,7 +577,7 @@ paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
 	else
 		yofs = -priv->yofs;
 
-	eog_debug_message (DEBUG_WINDOW, "zoom %.2f, xofs: %i, yofs: %i scaled w: %i h: %i\n", 
+	eog_debug_message (DEBUG_WINDOW, "zoom %.2f, xofs: %i, yofs: %i scaled w: %i h: %i\n",
 			   priv->zoom, xofs, yofs, scaled_width, scaled_height);
 
 	/* Draw background if necessary, in four steps */
@@ -606,8 +634,8 @@ paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
 	r.x1 = xofs + scaled_width;
 	r.y1 = yofs + scaled_height;
 
-	art_irect_intersect (&d, &r, rect);
-	if (art_irect_empty (&d))
+	eog_irect_intersect (&d, &r, rect);
+	if (eog_irect_empty (&d))
 		return;
 
 	switch (interp_type) {
@@ -618,7 +646,7 @@ paint_rectangle (EogScrollView *view, ArtIRect *rect, GdkInterpType interp_type)
 		str = "ALIASED";
 	}
 
-	eog_debug_message (DEBUG_WINDOW, "%s: x0: %i,\t y0: %i,\t x1: %i,\t y1: %i\n", 
+	eog_debug_message (DEBUG_WINDOW, "%s: x0: %i,\t y0: %i,\t x1: %i,\t y1: %i\n",
 			   str, d.x0, d.y0, d.x1, d.y1);
 
 	/* Short-circuit the fast case to avoid a memcpy() */
@@ -717,7 +745,7 @@ paint_iteration_idle (gpointer data)
 {
 	EogScrollView *view;
 	EogScrollViewPrivate *priv;
-	ArtIRect rect;
+	EogIRect rect;
 
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
@@ -726,12 +754,18 @@ paint_iteration_idle (gpointer data)
 
 	pull_rectangle (priv->uta, &rect, PAINT_RECT_WIDTH, PAINT_RECT_HEIGHT);
 
-	if (art_irect_empty (&rect)) {
-		art_uta_free (priv->uta);
+	if (eog_irect_empty (&rect)) {
+		eog_uta_free (priv->uta);
 		priv->uta = NULL;
-	} else
-		paint_rectangle (view, &rect, priv->interp_type);
-
+	} else {
+		if (is_zoomed_in (view))
+			paint_rectangle (view, &rect, priv->interp_type_in);
+		else if (is_zoomed_out (view))
+			paint_rectangle (view, &rect, priv->interp_type_out);
+		else
+			paint_rectangle (view, &rect, GDK_INTERP_NEAREST);
+	}
+		
 	if (!priv->uta) {
 		priv->idle_id = 0;
 		return FALSE;
@@ -748,11 +782,11 @@ static void
 request_paint_area (EogScrollView *view, GdkRectangle *area)
 {
 	EogScrollViewPrivate *priv;
-	ArtIRect r;
+	EogIRect r;
 
 	priv = view->priv;
 
-	eog_debug_message (DEBUG_WINDOW, "x: %i, y: %i, width: %i, height: %i\n", 
+	eog_debug_message (DEBUG_WINDOW, "x: %i, y: %i, width: %i, height: %i\n",
 			   area->x, area->y, area->width, area->height);
 
 	if (!GTK_WIDGET_DRAWABLE (priv->display))
@@ -769,7 +803,8 @@ request_paint_area (EogScrollView *view, GdkRectangle *area)
 		return;
 
 	/* Do nearest neighbor, 1:1 zoom or active progressive loading synchronously for speed.  */
-	if (priv->interp_type == GDK_INTERP_NEAREST ||
+	if ((is_zoomed_in (view) && priv->interp_type_in == GDK_INTERP_NEAREST) ||
+	    (is_zoomed_out (view) && priv->interp_type_out == GDK_INTERP_NEAREST) ||
 	    is_unity_zoom (view) ||
 	    priv->progressive_state == PROGRESSIVE_LOADING) {
 		paint_rectangle (view, &r, GDK_INTERP_NEAREST);
@@ -852,8 +887,8 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 
 	/* Ensure that the uta has the full size */
 
-	twidth = (width + ART_UTILE_SIZE - 1) >> ART_UTILE_SHIFT;
-	theight = (height + ART_UTILE_SIZE - 1) >> ART_UTILE_SHIFT;
+	twidth = (width + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
+	theight = (height + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
 
 	if (priv->uta)
 		g_assert (priv->idle_id != 0);
@@ -955,7 +990,7 @@ static void
 set_minimum_zoom_factor (EogScrollView *view)
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
-	
+
 	view->priv->min_zoom = MAX (1.0 / gdk_pixbuf_get_width (view->priv->pixbuf),
 				    MAX(1.0 / gdk_pixbuf_get_height (view->priv->pixbuf),
 					MIN_ZOOM_FACTOR) );
@@ -1180,13 +1215,6 @@ display_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 		zoom = 1.0;
 		break;
 
-	case GDK_F:
-	case GDK_f:
-		set_zoom_fit (view);
-		check_scrollbar_visibility (view, NULL);
-		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
-		break;
-
 	default:
 		return FALSE;
 	}
@@ -1224,7 +1252,7 @@ eog_scroll_view_button_press_event (GtkWidget *widget, GdkEventButton *event, gp
 	switch (event->button) {
 		case 1:
 		case 2:
-                        if (event->button == 1 && !priv->scroll_wheel_zoom && 
+                        if (event->button == 1 && !priv->scroll_wheel_zoom &&
 			    !(event->state & GDK_CONTROL_MASK))
 				break;
 
@@ -1520,7 +1548,7 @@ image_loading_update_cb (EogImage *img, int x, int y, int width, int height, gpo
 	view = (EogScrollView*) data;
 	priv = view->priv;
 
-	eog_debug_message (DEBUG_IMAGE_LOAD, "x: %i, y: %i, width: %i, height: %i\n", 
+	eog_debug_message (DEBUG_IMAGE_LOAD, "x: %i, y: %i, width: %i, height: %i\n",
 			   x, y, width, height);
 
 	if (priv->pixbuf == NULL) {
@@ -1567,7 +1595,7 @@ image_loading_finished_cb (EogImage *img, gpointer data)
 	else if (priv->interp_type != GDK_INTERP_NEAREST &&
 		 !is_unity_zoom (view))
 	{
-		// paint antialiased image version 
+		// paint antialiased image version
 		priv->progressive_state = PROGRESSIVE_POLISHING;
 		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 	}
@@ -1666,7 +1694,7 @@ eog_scroll_view_set_zoom_upscale (EogScrollView *view, gboolean upscale)
 }
 
 void
-eog_scroll_view_set_antialiasing (EogScrollView *view, gboolean state)
+eog_scroll_view_set_antialiasing_in (EogScrollView *view, gboolean state)
 {
 	EogScrollViewPrivate *priv;
 	GdkInterpType new_interp_type;
@@ -1677,8 +1705,26 @@ eog_scroll_view_set_antialiasing (EogScrollView *view, gboolean state)
 
 	new_interp_type = state ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST;
 
-	if (priv->interp_type != new_interp_type) {
-		priv->interp_type = new_interp_type;
+	if (priv->interp_type_in != new_interp_type) {
+		priv->interp_type_in = new_interp_type;
+		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
+	}
+}
+
+void
+eog_scroll_view_set_antialiasing_out (EogScrollView *view, gboolean state)
+{
+	EogScrollViewPrivate *priv;
+	GdkInterpType new_interp_type;
+
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+
+	priv = view->priv;
+
+	new_interp_type = state ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST;
+
+	if (priv->interp_type_out != new_interp_type) {
+		priv->interp_type_out = new_interp_type;
 		gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 	}
 }
@@ -1830,7 +1876,7 @@ eog_scroll_view_get_zoom_is_min (EogScrollView *view)
 
 	set_minimum_zoom_factor (view);
 
-	return DOUBLE_EQUAL (view->priv->zoom, MIN_ZOOM_FACTOR) || 
+	return DOUBLE_EQUAL (view->priv->zoom, MIN_ZOOM_FACTOR) ||
 	       DOUBLE_EQUAL (view->priv->zoom, view->priv->min_zoom);
 }
 
@@ -1876,8 +1922,8 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 			gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 
 		}
-		else if (priv->interp_type != GDK_INTERP_NEAREST &&
-			 !is_unity_zoom (view))
+		else if ((is_zoomed_in (view) && priv->interp_type_in != GDK_INTERP_NEAREST) ||
+			 (is_zoomed_out (view) && priv->interp_type_out != GDK_INTERP_NEAREST))
 		{
 			/* paint antialiased image version */
 			priv->progressive_state = PROGRESSIVE_POLISHING;
@@ -1887,14 +1933,14 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 		priv->image_changed_id = g_signal_connect (image, "changed",
 							   (GCallback) image_changed_cb, view);
 	}
-	
+
 	priv->image = image;
 }
 
-gboolean 
+gboolean
 eog_scroll_view_scrollbars_visible (EogScrollView *view)
 {
-	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (view->priv->hbar)) && 
+	if (!GTK_WIDGET_VISIBLE (GTK_WIDGET (view->priv->hbar)) &&
 	    !GTK_WIDGET_VISIBLE (GTK_WIDGET (view->priv->vbar)))
 		return FALSE;
 
@@ -1918,7 +1964,8 @@ eog_scroll_view_init (EogScrollView *view)
 	priv->zoom_mode = ZOOM_MODE_FIT;
 	priv->upscale = FALSE;
 	priv->uta = NULL;
-	priv->interp_type = GDK_INTERP_BILINEAR;
+	priv->interp_type_in = GDK_INTERP_BILINEAR;
+	priv->interp_type_out = GDK_INTERP_BILINEAR;
 	priv->scroll_wheel_zoom = FALSE;
 	priv->zoom_multiplier = IMAGE_VIEW_ZOOM_MULTIPLIER;
 	priv->image = NULL;
@@ -1940,9 +1987,9 @@ eog_scroll_view_dispose (GObject *object)
 
 	view = EOG_SCROLL_VIEW (object);
 	priv = view->priv;
-	
+
 	if (priv->uta != NULL) {
-		art_uta_free (priv->uta);
+		eog_uta_free (priv->uta);
 		priv->uta = NULL;
 	}
 
@@ -1983,6 +2030,55 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 	g_type_class_add_private (klass, sizeof (EogScrollViewPrivate));
 }
 
+static void
+view_on_drag_begin_cb (GtkWidget        *widget,
+		       GdkDragContext   *context,
+		       gpointer          user_data)
+{
+	EogScrollView *view;
+	EogImage *image;
+	GdkPixbuf *thumbnail;
+	gint width, height;
+
+	view = EOG_SCROLL_VIEW (user_data);
+	image = view->priv->image;
+
+	thumbnail = eog_image_get_thumbnail (image);
+
+	if  (thumbnail) {
+		width = gdk_pixbuf_get_width (thumbnail);
+		height = gdk_pixbuf_get_height (thumbnail);
+		gtk_drag_set_icon_pixbuf (context, thumbnail, width/2, height/2);
+		g_object_unref (thumbnail);
+	}
+}
+
+static void
+view_on_drag_data_get_cb (GtkWidget        *widget,
+			  GdkDragContext   *drag_context,
+			  GtkSelectionData *data,
+			  guint             info,
+			  guint             time,
+			  gpointer          user_data)
+{
+	EogScrollView *view;
+	EogImage *image;
+	gchar *uris[2];
+	GFile *file;
+
+	view = EOG_SCROLL_VIEW (user_data);
+
+	image = view->priv->image;
+
+	file = eog_image_get_file (image);
+	uris[0] = g_file_get_uri (file);
+	uris[1] = NULL;
+
+	gtk_selection_data_set_uris (data, uris);
+
+	g_free (uris[0]);
+	g_object_unref (file);
+}
 
 GtkWidget*
 eog_scroll_view_new (void)
@@ -2039,6 +2135,14 @@ eog_scroll_view_new (void)
 
 	g_signal_connect (G_OBJECT (widget), "key_press_event", G_CALLBACK (display_key_press_event), view);
 
+	gtk_drag_source_set (priv->display, GDK_BUTTON1_MASK,
+			     target_table, G_N_ELEMENTS (target_table),
+			     GDK_ACTION_COPY);
+	g_signal_connect (G_OBJECT (priv->display), "drag-data-get",
+			  G_CALLBACK (view_on_drag_data_get_cb), widget);
+	g_signal_connect (G_OBJECT (priv->display), "drag-begin",
+			  G_CALLBACK (view_on_drag_begin_cb), widget);
+
 	gtk_table_attach (table, priv->display,
 			  0, 1, 0, 1,
 			  GTK_EXPAND | GTK_FILL,
@@ -2055,18 +2159,18 @@ eog_scroll_view_new (void)
 			  0, 0);
 
 	gtk_widget_show_all (widget);
-	
+
 	return widget;
 }
 
-static void 
+static void
 eog_scroll_view_popup_menu (EogScrollView *view, GdkEventButton *event)
 {
 	GtkWidget *popup;
 	int button, event_time;
 
 	popup = view->priv->menu;
-	
+
 	if (event) {
 		button = event->button;
 		event_time = event->time;
@@ -2074,15 +2178,15 @@ eog_scroll_view_popup_menu (EogScrollView *view, GdkEventButton *event)
 		button = 0;
 		event_time = gtk_get_current_event_time ();
 	}
-	
-	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL, 
+
+	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
 			button, event_time);
 }
 
 static gboolean
-view_on_button_press_event_cb (GtkWidget *view, GdkEventButton *event, 
+view_on_button_press_event_cb (GtkWidget *view, GdkEventButton *event,
 			       gpointer user_data)
-{    
+{
     /* Ignore double-clicks and triple-clicks */
     if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
     {
@@ -2090,7 +2194,7 @@ view_on_button_press_event_cb (GtkWidget *view, GdkEventButton *event,
 
 	    return TRUE;
     }
-    
+
     return FALSE;
 }
 
@@ -2103,8 +2207,8 @@ eog_scroll_view_set_popup (EogScrollView *view,
 
 	view->priv->menu = g_object_ref (menu);
 
-	gtk_menu_attach_to_widget (GTK_MENU (view->priv->menu), 
-				   GTK_WIDGET (view), 
+	gtk_menu_attach_to_widget (GTK_MENU (view->priv->menu),
+				   GTK_WIDGET (view),
 				   NULL);
 
 	g_signal_connect (G_OBJECT (view), "button_press_event",
@@ -2121,7 +2225,7 @@ eog_scroll_view_set_bg_color (EogScrollView *view,
 }
 
 void
-eog_scroll_view_set_scroll_wheel_zoom (EogScrollView *view, 
+eog_scroll_view_set_scroll_wheel_zoom (EogScrollView *view,
 				       gboolean       scroll_wheel_zoom)
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
@@ -2130,7 +2234,7 @@ eog_scroll_view_set_scroll_wheel_zoom (EogScrollView *view,
 }
 
 void
-eog_scroll_view_set_zoom_multiplier (EogScrollView *view, 
+eog_scroll_view_set_zoom_multiplier (EogScrollView *view,
 				     gdouble        zoom_multiplier)
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
